import path from "path";
import fs from "fs/promises";
import Cloudformation from "aws-sdk/clients/cloudformation.js";
export async function manifest(root) {
    const manifestPath = path.join(root, ".build", "cdk.out", "manifest.json");
    const data = await fs.readFile(manifestPath);
    return JSON.parse(data.toString());
}
export async function metadata(root, config) {
    const man = await manifest(root);
    const stacks = Object.values(man.artifacts).filter(a => a.type === "aws:cloudformation:stack");
    const constructs = await Promise.all(stacks.map(async (stack) => {
        const region = stack.environment.split("/").pop();
        const cfn = new Cloudformation({ region });
        const resource = await cfn
            .describeStackResource({
            StackName: stack.properties.stackName || stack.displayName,
            LogicalResourceId: "SSTMetadata"
        })
            .promise();
        const parsed = JSON.parse(resource.StackResourceDetail.Metadata);
        const constructs = parsed["sst:constructs"];
        return constructs;
    }));
    return constructs.flat();
}
